/*****************************************************************************
 * c2html:C++ utility to convert C/C++ code to HTML with syntax highlighting.* 
 * Author:Ankit Rohatgi <ankit_rohatgi@yahoo.com>                            *
 *        http://on.to/ank                                                   *
 *                                                                           *
 *****************************************************************************/ 
#define STYLENO 3
#define STYLELENGTH 20
char C_COMMENT[STYLENO][20];
char C_NORMAL[STYLENO][STYLELENGTH];
char C_SYMBOL[STYLENO][STYLELENGTH];
char C_TYPE[STYLENO][STYLELENGTH];
char C_STATEMENT[STYLENO][STYLELENGTH];
char C_CONSTANT[STYLENO][STYLELENGTH];
char C_PREPROC[STYLENO][STYLELENGTH];
char C_STRING[STYLENO][STYLELENGTH];
char C_BG[20];
char C_FG[20];

#include "c2html.h"
#define MAXPATH 100 
//Function declarations
void usage_note();
void select_style(char *style);
void style_break(char stl[],char label[],char fg[],char bg[],char typ[]);
void set_defaults();

int main(int argc, char *argv[])
{
   char fln_in[MAXPATH],fln_out[MAXPATH];
   set_defaults();
	
   if(argc<3)
   { usage_note(); return(1);
   }

	int inp=0,op=0;
   for(int i=1;i<argc;i++)
   {
	   if(argv[i][0]=='-' && argv[i][1]=='-')
	   {
		   if(strcmp(argv[i],"--style")==0)
		   {
			   if(i<argc-1 && argv[i+1][0]!='-'){select_style(argv[++i]);}
			   else {cout<<"No style specified !";}
		   }
		   else if(strcmp(argv[i],"--help")==0)
		   {
			   usage_note();
		   }
		   else
		   {
			   cerr<<"\nUnknown Option: "<<argv[i];
		   }	 
				  
	   }
	   else if(argv[i][0]=='-')
	   {
		   if(strcmp(argv[i],"-h")==0)
		   {
			   usage_note();
		   }	
	   }
	   else
	   {
		   if(inp==0 && op==0)
		   {
			   strcpy(fln_in,argv[i]); inp=1;
		   }
	       else if(op==0)
		   {
			   strcpy(fln_out,argv[i]); op=1;
		   }
		   else
		   {
			   cout<<"I don't know what "<<argv[i]<<" is !";
		   }
	   }
   }
	   
   if(inp==1 && op==1)
   {	   
	   readcode rc;
	   rc.open(fln_in,fln_out);
	   rc.reader();
   }
   cout<<"I'm done !\n";
   return(0);
}      

//************ Note on usage ******************
void usage_note()
{ 
	cout<<"\nC2html: Copyright (C), Ankit Rohatgi, 2002"
		<<"\nConverts C/C++ source to HTML with syntax highlighting."
		<<"\nUsage: $c2html <c/c++ file>  <output html file>\n";
}
//********  select_style() ****************	   	

void select_style(char *style)
{
	ifstream fl("styles.c2h");
	if(!fl)
	{ cout<<"styles.c2h not found !"; }
	char temp[80],t2[80],ch,s; int i,f=0,go=1;;
	while(fl.getline(temp,80) && f==0)
	{
		if(temp[0]!='#') //only if not a comment
		{
			if(temp[0]=='[')//style name
			{ 
				cout<<"gotcha !";
				for(i=0;i<strlen(temp)-2;i++)
				{
					t2[i]=temp[i+1];
				}
				t2[i]='\0';
				if(strcmp(t2,style)==0)
				{
					f=1;
					while(fl.getline(temp,80) && go==1)
					{  
						if(isalnum(temp[0]))
						{
							char label[80],fg[20],bg[20],typ[20];
						    style_break(temp,label,fg,bg,typ);
							if(strcmp(label,"comment")==0)
							{
								strcpy(C_COMMENT[0],fg);
								strcpy(C_COMMENT[1],bg);
								strcpy(C_COMMENT[2],typ);
							}
							else if(strcmp(label,"symbol")==0)
							{
								// symbol colors
								strcpy(C_SYMBOL[0],fg);
								strcpy(C_SYMBOL[1],bg);
								strcpy(C_SYMBOL[2],typ);
							}
							else if(strcmp(label,"normal")==0)
							{
								//normal
								strcpy(C_NORMAL[0],fg);
								strcpy(C_NORMAL[1],bg);
								strcpy(C_NORMAL[2],typ);
							}
							else if(strcmp(label,"constant")==0)
							{
								// constant
								strcpy(C_CONSTANT[0],fg);
								strcpy(C_CONSTANT[1],bg);
								strcpy(C_CONSTANT[2],typ);
							}
							else if(strcmp(label,"string")==0)
							{
								// string
								strcpy(C_STRING[0],fg);
								strcpy(C_STRING[1],bg);
								strcpy(C_STRING[2],typ);
							}
							else if(strcmp(label,"datatype")==0)
							{
								//type
								strcpy(C_TYPE[0],fg);
								strcpy(C_TYPE[1],bg);
								strcpy(C_TYPE[2],typ);
							}
							else if(strcmp(label,"statement")==0)
							{
								//statements
								strcpy(C_STATEMENT[0],fg);
								strcpy(C_STATEMENT[1],bg);
								strcpy(C_STATEMENT[2],typ);
							}
							else if(strcmp(label,"preprocessor")==0)
							{
								// preproc
								strcpy(C_PREPROC[0],fg);
								strcpy(C_PREPROC[1],bg);
								strcpy(C_PREPROC[2],typ);
							}
							else if(strcmp(label,"background")==0)
							{
								strcpy(C_BG,fg);
							}
							else if(strcmp(label,"foreground")==0)
							{
								strcpy(C_FG,fg);
							}
						}
						else if(temp[0]=='[')
						{
							go=0;
						}
					}
				}
			}
		}
	}
}
					
void style_break(char stl[],char label[],char fg[],char bg[],char typ[])
{
	char tstr[80];
	strcpy(label,""); 
	strcpy(fg,""); 
	strcpy(bg,""); 
	strcpy(typ,""); 
	char ch[2]; ch[1]='\0';
	int f=0,i=0,d=0,t=0;
	while(i<strlen(stl) && d==0)
	{
		if(stl[i]!=' ' && stl[i]!='\n' 
				&& stl[i]!='\t' && stl[i]!=',' && stl[i]!='.')
		{
			tstr[t++]=stl[i];
		}
		else if(stl[i]==','||stl[i]=='.') 
		{
			tstr[t]='\0';
			switch(f)
			{   case 0: strcpy(label,tstr);break;
			    case 1: strcpy(fg,tstr);break;
				case 2: strcpy(bg,tstr); break;
				case 3: strcpy(typ,tstr);break;
				default:d=1;break;
			}
		    f++; t=0;
		    if(stl[i]=='.') { d=1; }
		    	
	     }
		 i++;
	}
}
   
void set_defaults()
{

   
	strcpy(C_FG,"lime"); 
	strcpy(C_BG,"darkblue");
    
	strcpy(C_COMMENT[0],"lightblue"); 
	strcpy(C_COMMENT[1],""); 
	strcpy(C_COMMENT[2],""); 
    
	strcpy(C_NORMAL[0],"lime");
	strcpy(C_NORMAL[1],"");
	strcpy(C_NORMAL[2],"");
	
    strcpy(C_SYMBOL[0],"yellow");
    strcpy(C_SYMBOL[1],"");
    strcpy(C_SYMBOL[2],"");
	
    strcpy(C_TYPE[0],"white");
    strcpy(C_TYPE[1],"");
    strcpy(C_TYPE[2],"bold");
	
    strcpy(C_STATEMENT[0],"pink");
    strcpy(C_STATEMENT[1],"");
    strcpy(C_STATEMENT[2],"bold");
	
    strcpy(C_CONSTANT[0],"silver");
    strcpy(C_CONSTANT[1],"");
    strcpy(C_CONSTANT[2],"");
	
    strcpy(C_PREPROC[0],"darkblue");
    strcpy(C_PREPROC[1],"lightblue");
    strcpy(C_PREPROC[2],"");
	
    strcpy(C_STRING[0],"red");
    strcpy(C_STRING[1],"");
    strcpy(C_STRING[2],"");
	// count types:
	int i=0;
	while(strcmp(datatypes[i],"")!=0)
		i++;
	TYPES=i; 
	i=0;
	while(strcmp(statements[i],"")!=0)
		i++;
	STATEMENTS=i;
}
							
		 
